@@ -619,7 +619,7 @@ spawn_session (State        *state,
                 for (i = 0; state->environment[i] != NULL; i++) {
                         g_auto(GStrv) environment_entry = NULL;
 
-                        if (state->environment[i] == '\0') {
+                        if (state->environment[i][0] == '\0') {
                                 continue;
                         }
 
@@ -643,6 +643,8 @@ spawn_session (State        *state,
                 g_subprocess_launcher_setenv (launcher, "WINDOWPATH", vt, TRUE);
         }
 
+        g_subprocess_launcher_unsetenv (launcher, "GNOME_SHELL_SESSION_MODE");
+
         if (run_script) {
                 subprocess = g_subprocess_launcher_spawn (launcher,
                                                           &error,
